/*
 * Copyright (c) Saga Inc.
 * Distributed under the terms of the GNU Affero General Public License v3.0 License.
 */

.text-and-icon-button {
  display: flex;
  align-items: center;
  justify-content: space-between;
  gap: 5px;
}

.text-and-icon-button__icon {
  height: 100%;
  display: flex;
  align-items: center;
}

.text-and-icon-button__icon > svg {
  height: 100%;
  width: auto;
}
