"use strict";(self["webpackChunk_jupyterlab_application_top"]=self["webpackChunk_jupyterlab_application_top"]||[]).push([[3048],{53048:(r,a,e)=>{e.d(a,{diagram:()=>_});var s=e(41359);var t=e(15051);var l=e(94065);var p=e(33416);var v=e(94746);var i=e(20778);var n=e(57590);var u=e(68232);var c=e(76261);var o=e(96049);var b=e(75905);var _={parser:s._$,get db(){return new s.NM},renderer:s.Lh,styles:s.tM,init:(0,b.K2)((r=>{if(!r.class){r.class={}}r.class.arrowMarkerAbsolute=r.arrowMarkerAbsolute}),"init")}}}]);