"use strict";(self["webpackChunk_jupyterlab_application_top"]=self["webpackChunk_jupyterlab_application_top"]||[]).push([[9136],{49136:(a,e,p)=>{p.d(e,{createInfoServices:()=>t.v});var t=p(25996);var c=p(74888)}}]);