"use strict";(self["webpackChunk_jupyterlab_application_top"]=self["webpackChunk_jupyterlab_application_top"]||[]).push([[3122],{63122:(a,e,p)=>{p.d(e,{createPacketServices:()=>t.$});var t=p(69602);var c=p(74888)}}]);