"use strict";(self["webpackChunk_jupyterlab_application_top"]=self["webpackChunk_jupyterlab_application_top"]||[]).push([[8354],{68354:(a,p,e)=>{e.d(p,{createGitGraphServices:()=>t.b});var t=e(87290);var r=e(74888)}}]);