/*
 * Copyright (c) Saga Inc.
 * Distributed under the terms of the GNU Affero General Public License v3.0 License.
 */

.jp-ToolbarButtonComponent > svg[data-icon='lightbulb-icon'] {
  color: #9c6bff;
}
