"use strict";(self["webpackChunk_jupyterlab_application_top"]=self["webpackChunk_jupyterlab_application_top"]||[]).push([[7741],{97741:(a,e,p)=>{p.d(e,{createRadarServices:()=>t.f});var t=p(36578);var r=p(74888)}}]);