/*
 * Copyright (c) Saga Inc.
 * Distributed under the terms of the GNU Affero General Public License v3.0 License.
 */

.markdown-message-part * .jp-RenderedHTMLCommon :not(pre) > code {
    background-color: var(--purple-300);
    color: var(--purple-700);
    border-radius: 3px;
    display: inline-flex;
    align-items: center;
}

/* Ensure long URLs in markdown messages wrap instead of overflowing */
.markdown-message-part a {
    word-break: break-all;
    overflow-wrap: anywhere;
    white-space: pre-line;
}