"use strict";(self["webpackChunk_jupyterlab_application_top"]=self["webpackChunk_jupyterlab_application_top"]||[]).push([[9359],{79359:(e,a,p)=>{p.d(a,{createArchitectureServices:()=>t.S});var t=p(77018);var c=p(74888)}}]);