/*
 * Copyright (c) Saga Inc.
 * Distributed under the terms of the GNU Affero General Public License v3.0 License.
 */

/*
    See the JupyterLab Developer Guide for useful CSS Patterns:

    https://jupyterlab.readthedocs.io/en/stable/developer/css.html
*/

@import url('icons.css');
@import url('statusItem.css');

:root {
  --chat-background-color: var(--jp-layout-color1);
  --chat-user-message-background-color: var(--jp-input-background);
  --chat-user-message-font-color: var(--jp-content-font-color1);
  --chat-assistant-message-font-color: #0e1119;
  --muted-text-color: var(--jp-content-font-color2);

  --green-900: #155239;
  --green-800: #1a7741;
  --green-700: #239d58;
  --green-600: #39c172;
  --green-500: #74d99e;
  --green-400: #a8eec1;
  --green-300: #e3fcec;

  --red-900: #611818;
  --red-800: #891a1b;
  --red-700: #b82020;
  --red-600: #db3130;
  --red-500: #e46464;
  --red-400: #f5aaaa;
  --red-300: #fce8e8;

  --yellow-900: #5c4813;
  --yellow-500: #fae29f;
  --yellow-300: #fde047;
  --yellow-100: #fef9c3;

  --grey-900: #222934;
  --grey-800: #5f6b7a;
  --grey-700: #8795a7;
  --grey-500: #b8c4ce;
  --grey-400: #cfd6de;
  --grey-300: #e1e7eb;
  --grey-200: #f8f9fa;

  --purple-700: #844af7;
  --purple-600: #9d6cff;
  --purple-500: #ba9bf8;
  --purple-400: #d0b9fe;
  --purple-300: #e9e0fd;
}

.text-muted {
  color: var(--jp-ui-font-color3);
}

.text-sm {
  font-size: 12px;
}

/* 
    Jupyter tries to ensure that the cell toolbar does not overlap with the contents of the cell. 
    There's some algorithm that adds the class `.jp-toolbar-overlap` to the cell toolbar 
    when it gets too close to the cell contents. 

    However, we don't want to hide the Accept and Reject toolbar buttons! We always want them to be visible.
    To do this, we change the method for hiding the cell toolbar. Instead of hiding the entire toolbar, 
    we hide individual buttons, excluding the Accept and Reject buttons.
*/
.jp-toolbar-overlap .jp-cell-toolbar {
  display: flex !important;
}

.jp-toolbar-overlap
  .jp-cell-toolbar
  > *:not([data-jp-item-name='accept-code']):not(
    [data-jp-item-name='reject-code']
  ) {
  display: none !important;
}