/*
 * Copyright (c) Saga Inc.
 * Distributed under the terms of the GNU Affero General Public License v3.0 License.
 */

.text-button-mito-ai {
  display: flex;
  align-items: center;
  justify-content: space-around;
}
